#!/bin/sh

USERADD=`which useradd`
USERDEL=`which userdel`
GROUPDEL=`which groupdel`
GROUPADD=`which groupadd`
USERMOD=`which usermod`
GROUPMOD=`which groupmod`
NEWUSERS=`which newusers`
CHPASSWD=`which chpasswd`
CHGPASSWD=`which chgpasswd`
CPPW=`which cppw`
GRPCK=`which grpck`
PWCK=`which pwck`


USER_NAME="username"
GROUP_NAME="usergroup"
FULL_NAME="User Full Name"
PSWD="qwe123"
NEW_PSWD="qwe456"
EN_PSWD=`openssl passwd -crypt $PSWD`
NEW_EN_PSWD=`openssl passwd -crypt "$PSWD"`
NEW_FULL_NAME="New Full Name of User"
NEW_HOME_DIR="/home/new_dir$USER_NAME"

PASSWD_FILE=/etc/passwd
GROUP_FILE=/etc/group
TMP_PASSWD_FILE=/tmp/tmp_passwd
TMP_GROUP_FILE=/tmp/tmp_group
PASS_TEST_CASE=0

PWD_NF=7
GRP_NF=4

# Additional function to check existance of group or user
# $1 - user or group name
# $2 - /etc/passwd or /etc/group file
check_nonexistance() 
{
	if [ `grep "^$1:" $2 -c` -gt 0 ]  ; then
		echo "User or group with $1 name already exists. Return 1"
		return 1
	fi
}

# Additional function to test correct execution of input command
assert_cmd() 
{
	echo "$TEST_NAME: Launching $*"
	eval $*
	if [ $? -ne 0 ] ; then 
		echo "$TEST_NAME: Error occures during $*"
		exit_cmd 
	fi
}

exit_cmd()
{
	del_modification
	echo "$TEST_NAME: TEST FAILED. EXIT"
	exit 1
}

# Additional function to clear user & group files
del_modification()
{
	(check_nonexistance $USER_NAME $PASSWD_FILE) || (($USERDEL $USER_NAME)&&(echo "$TEST_NAME: User $USER_NAME was deleted" ))	 
	(check_nonexistance $GROUP_NAME $GROUP_FILE) || (($GROUPDEL $GROUP_NAME) && (echo "$TEST_NAME: Ggroup $GROUP_NAME was deleted"))
	rm -f $TMP_PASSWD_FILE
	rm -f $TMP_GROUP_FILE
}

# Additional function which change the file
modify_file()
{
	echo "$TEST_NAME: Replace $1 with $2 in $3"
	return `sed -i "s/^$1:/$2:/g"  $3`
}


# Check that /etc/passwd has the appropriate structure
# Check the condition giving by param
# $1 - condition to check
check_record() 
{
	FILE=$1
	NF=$2
	shift 2

	echo "$TEST_NAME: Check record in $FILE file"
	RECORD=`grep "^$1:" $FILE`
	echo "$TEST_NAME: String from $FILE is: $RECORD"
	echo "$TEST_NAME: String for compare is: $*"
	cnt=`echo $RECORD | awk '{print split($0, myarr, ":")}'`

	if [ $cnt -ne $NF ] ; then
		echo "$TEST_NAME: Utility writes $cnt field number in $FILE file instead of $NF"
		return 1
	fi

	j=1
	for i in "$@"
	do
		if [ "$i" ] ; then
			user_data=`echo $RECORD | awk '{split($0, myarr, ":"); print myarr[j]}' j=$j`
			echo "$TEST_NAME: $i ? $user_data"
			if [ "$i" != "$user_data" ] ; then
				echo "$TEST_NAME: User data: $user_data; Parameter: $i"
				echo "$TEST_NAME: Utility writes unexpected data in $FILE file. Error"
				return 1
			fi
		fi
		let j+=1
	done
}

# Test useradd/userdel, groupadd, groupdel 
test_passwd_pkg()
{
# Start tes cases
	TEST_NAME=PASSWD_PKG_CHECK
	del_modification

	# groupadd
	echo "$TEST_NAME: groupadd TEST"
	assert_cmd "$GROUPADD $GROUP_NAME"
	assert_cmd check_record '$GROUP_FILE $GRP_NF  $GROUP_NAME'
	echo "$TEST_NAME:  PASS"

	# useradd
	echo "$TEST_NAME: useradd TEST"
	assert_cmd '$USERADD -p $EN_PSWD -c "$FULL_NAME" -g $GROUP_NAME $USER_NAME'
	salt=`grep "^$USER_NAME:"  $PASSWD_FILE | awk '{split($0, myarr, ":"); print substr(myarr[2], 1, 2)}'`
	group_id=`grep "^$GROUP_NAME:" $GROUP_FILE | awk '{split($0, myarr, ":"); print myarr[3]}'`
	calc_pswd=`openssl passwd -salt $salt $PSWD`
	user_str=`grep "^$USER_NAME:"  $PASSWD_FILE`
	assert_cmd check_record '"$PASSWD_FILE" "$PWD_NF " "$USER_NAME" "$calc_pswd" "" "$group_id" "$FULL_NAME" "/home/$USER_NAME" "/bin/sh"'
	echo "$TEST_NAME:  PASS"

	# groupmod
	echo "$TEST_NAME: groupmod TEST"
	NEW_GROUP_ID=`expr "$group_id" + 1`
	assert_cmd '$GROUPMOD -g "$NEW_GROUP_ID" -o  $GROUP_NAME'
	assert_cmd check_record '$GROUP_FILE $GRP_NF $GROUP_NAME "" "$NEW_GROUP_ID"'
	echo "$TEST_NAME:  PASS"

	# usermod
	echo "$TEST_NAME: usermod TEST"

	assert_cmd '$USERMOD -c "$NEW_FULL_NAME" -d "$NEW_HOME_DIR" -g $GROUP_NAME -p $NEW_EN_PSWD $USER_NAME'
	salt=`grep "^$USER_NAME:"  $PASSWD_FILE | awk '{split($0, myarr, ":"); print substr(myarr[2], 1, 2)}'`
	calc_pswd=`openssl passwd -salt $salt "$PSWD"`
	assert_cmd check_record '"$PASSWD_FILE" "$PWD_NF" "$USER_NAME" "$calc_pswd" "" "$NEW_GROUP_ID" "$NEW_FULL_NAME" "$NEW_HOME_DIR" "/bin/sh"'
	echo "$TEST_NAME:  PASS"

	# required copy action
	echo "$TEST_NAME: "
	assert_cmd "cat $PASSWD_FILE > $TMP_PASSWD_FILE"
	assert_cmd "cat $GROUP_FILE > $TMP_GROUP_FILE"

	# cppw & cpgp
	echo "$TEST_NAME: cppw TEST"
	assert_cmd $USERDEL $USER_NAME
	assert_cmd $CPPW $TMP_PASSWD_FILE
	assert_cmd check_record '"$PASSWD_FILE" "$PWD_NF" "$USER_NAME" "$calc_pswd" "" "$NEW_GROUP_ID" "$NEW_FULL_NAME" "$NEW_HOME_DIR" "/bin/sh"'
	echo "$TEST_NAME:  PASS"

	# grpck
	echo "$TEST_NAME: grpck TEST"
	assert_cmd modify_file $GROUP_NAME $GROUP_NAME: $GROUP_FILE
	($GRPCK -r $GROUP_FILE) && (exit_cmd)
	assert_cmd modify_file $GROUP_NAME: $GROUP_NAME $GROUP_FILE
	echo "$TEST_NAME:  PASS"

	# pwck
	echo "$TEST_NAME: pwck TEST"
	assert_cmd modify_file $USER_NAME $USER_NAME: $PASSWD_FILE
	($PWCK -r $PASSWD_FILE) && (exit_cmd)
	assert_cmd modify_file $USER_NAME: $USER_NAME $PASSWD_FILE
	echo "$TEST_NAME:  PASS"

	# chpasswd
	echo "$TEST_NAME: chpasswd TEST"
	assert_cmd 'echo "$USER_NAME:$NEW_PSWD" | $CHPASSWD '
	salt=`grep "^$USER_NAME:"  $PASSWD_FILE | awk '{split($0, myarr, ":"); print substr(myarr[2], 1, 2)}'`
	calc_pswd=`openssl passwd -salt $salt $NEW_PSWD`
	assert_cmd check_record '"$PASSWD_FILE" "$PWD_NF" "$USER_NAME" "$calc_pswd"'
	echo "$TEST_NAME:  PASS"

	# chgpasswd
	echo "$TEST_NAME: chgpasswd TEST"
	assert_cmd 'echo "$GROUP_NAME:$NEW_PSWD" | $CHGPASSWD '
	salt=`grep "^$GROUP_NAME:"  $GROUP_FILE | awk '{split($0, myarr, ":"); print substr(myarr[2], 1, 2)}'`
	calc_pswd=`openssl passwd -salt $salt $NEW_PSWD`
	assert_cmd check_record '$GROUP_FILE $GRP_NF "$GROUP_NAME" "$calc_pswd"'
	echo "$TEST_NAME:  PASS"

	echo `cat $PASSWD_FILE | grep $USER_NAME` > $TMP_PASSWD_FILE

	# userdel
	echo "$TEST_NAME: userdel TEST"
	assert_cmd $USERDEL $USER_NAME
	assert_cmd check_nonexistance $USER_NAME $PASSWD_FILE
	echo `cat $PASSWD_FILE | grep $PASSWD_FILE`
	echo "$TEST_NAME:  PASS"

	# groupdel
	echo "$TEST_NAME: groupdel TEST"
	assert_cmd "$GROUPDEL $GROUP_NAME"		
	assert_cmd check_nonexistance $GROUP_NAME $GROUP_FILE
	echo "$TEST_NAME:  PASS"

	# newusers	
	echo "$TEST_NAME: newusers TEST"
	assert_cmd $NEWUSERS $TMP_PASSWD_FILE
	assert_cmd $USERDEL $USER_NAME
	assert_cmd $GROUPDEL $USER_NAME
	echo "$TEST_NAME:  PASS"

	# end of test cases	
	del_modification
	echo
	echo "$TEST_NAME: TESTS PASSED"
}

# Execute test
test_passwd_pkg

